@@ -8,6 +8,7 @@ from functools import lru_cache
 import threading
 import time
 from datetime import datetime
+import torch
 
 import numpy as np
 from sentence_transformers import SentenceTransformer
@@ -160,6 +161,10 @@ class SyntheticContrastivePairGenerator:
         Returns:
             list of question descriptions
         """
+        print(f"🔍 [generate_questions] Starting with trait: {trait_description}, num_questions: {num_questions}")
+        print(f"🔍 [generate_questions] self.model type: {type(self.model)}")
+        print(f"🔍 [generate_questions] self.model.tokenizer type: {type(self.model.tokenizer)}")
+        
         if self.model is None and force_new:
             raise ValueError("No model loaded. Cannot generate new questions.")
             
@@ -197,7 +202,15 @@ class SyntheticContrastivePairGenerator:
                 gap = max(num_questions - available_count, 50)  # At least 50 new questions
                 target_new = gap * 2  # Generate 2x what we need
             
-            generated_questions = self._generate_new_questions_parallel(target_new)
+            print(f"🔍 [generate_questions] About to call _generate_new_questions_parallel")
+            try:
+                generated_questions = self._generate_new_questions_parallel(target_new)
+                print(f"🔍 [generate_questions] _generate_new_questions_parallel returned {len(generated_questions)} questions")
+            except Exception as e:
+                print(f"🔍 [generate_questions] _generate_new_questions_parallel failed with: {e}")
+                import traceback
+                print(f"🔍 [generate_questions] Traceback:\n{traceback.format_exc()}")
+                raise
             
             # Add to bank
             added_count = self.question_bank.add_questions(generated_questions)
@@ -221,6 +234,10 @@ class SyntheticContrastivePairGenerator:
         Returns:
             List of generated questions
         """
+        print(f"🔍 [_generate_new_questions_parallel] Starting with target_count: {target_count}")
+        print(f"🔍 [_generate_new_questions_parallel] self.model type: {type(self.model)}")
+        print(f"🔍 [_generate_new_questions_parallel] self.model.tokenizer type: {type(self.model.tokenizer)}")
+        
         if self.model is None:
             raise ValueError("No model loaded. Cannot generate questions.")
             
@@ -244,29 +261,10 @@ class SyntheticContrastivePairGenerator:
             
             # Collect results
             for i, future in futures:
-                try:
-                    response = future.result(timeout=60)  # 60 second timeout
-                    questions = self._parse_questions_from_response(response)
-                    all_questions.extend(questions)
-                    print(f"   ✅ Template {i+1} generated {len(questions)} questions")
-                except Exception as e:
-                    import traceback
-                    print(f"   ⚠️ Error with template {i+1}: {e}")
-                    print(f"   Traceback: {traceback.format_exc()}")
-                    continue
-        
-        # If we got very few questions due to timeouts, try a simpler approach
-        if len(all_questions) < target_count // 2:
-            print(f"   ⚠️ Only got {len(all_questions)} questions from parallel generation, trying sequential fallback...")
-            # Try one more time with the simplest template
-            try:
-                simple_prompt = f"List {target_count} simple questions about everyday life:\n1."
-                response = self._cached_generate_response(simple_prompt, QUESTION_GEN.CONFIG)
-                fallback_questions = self._parse_questions_from_response(response)
-                all_questions.extend(fallback_questions)
-                print(f"   ✅ Fallback generated {len(fallback_questions)} additional questions")
-            except Exception as e:
-                print(f"   ⚠️ Fallback also failed: {e}")
+                response = future.result()
+                questions = self._parse_questions_from_response(response)
+                all_questions.extend(questions)
+                print(f"   ✅ Template {i+1} generated {len(questions)} questions")
         
         # Deduplicate
         with TimingContext(f"Deduplicating {len(all_questions)} questions", self.enable_timing):
@@ -340,8 +338,12 @@ class SyntheticContrastivePairGenerator:
         Returns:
             A list of unique question strings
         """
+        print(f"🔍 [_deduplicate_questions] Starting with {len(questions)} questions")
+        print(f"🔍 [_deduplicate_questions] self.similarity_model: {self.similarity_model}")
+        
         if not self.similarity_model or len(questions) <= 1:
             # Fallback to simple text-based deduplication
+            print(f"🔍 [_deduplicate_questions] Using text-based deduplication")
             return list(set(questions))
 
         # Batch encode all questions at once
@@ -433,16 +435,12 @@ class SyntheticContrastivePairGenerator:
             # Collect results
             successful_pairs = []
             for question, future in futures:
-                try:
-                    pair = future.result(timeout=60)  # 60 second timeout per pair
-                    if pair is not None:
-                        successful_pairs.append(pair)
-                        if current_count + len(successful_pairs) >= target_count:
-                            print(f"✅ Reached target of {target_count} pairs for selection")
-                            break
-                except Exception as e:
-                    print(f"   ⚠️ Failed to generate pair for question '{question[:50]}...': {e}")
-                    continue
+                pair = future.result()
+                if pair is not None:
+                    successful_pairs.append(pair)
+                    if current_count + len(successful_pairs) >= target_count:
+                        print(f"✅ Reached target of {target_count} pairs for selection")
+                        break
         
         return successful_pairs
     
@@ -565,46 +563,92 @@ class SyntheticContrastivePairGenerator:
         # Extract layer_index from config (required parameter for Model.generate)
         layer_index = merged_config.pop('layer_index', 15)  # Default to layer 15
         
-        # Remove enable_thinking as it should be set on model config, not passed to generate
-        merged_config.pop('enable_thinking', None)
+        # Extract enable_thinking to pass to format_prompt
+        enable_thinking = merged_config.pop('enable_thinking', None)
         
-        # Add thinking tokens to suppress_tokens if needed
-        if self.suppress_thinking_tokens:
-            # Token IDs for <think> and </think> in Qwen models
-            think_tokens = [151667, 151668]  # <think> and </think>
-            existing_suppress = merged_config.get('suppress_tokens', [])
-            if isinstance(existing_suppress, list):
-                merged_config['suppress_tokens'] = existing_suppress + think_tokens
+        # Format the prompt with model-specific parameters
+        if enable_thinking is not None:
+            formatted_prompt = self.model.format_prompt(prompt, enable_thinking=enable_thinking)
+        else:
+            formatted_prompt = self.model.format_prompt(prompt)
+        
+        # Generate using the formatted prompt directly
+        # Debug logging
+        import threading
+        print(f"🔍 [Thread {threading.current_thread().name}] About to tokenize")
+        print(f"🔍 [Thread {threading.current_thread().name}] self.model type: {type(self.model)}")
+        print(f"🔍 [Thread {threading.current_thread().name}] self.model.tokenizer type: {type(self.model.tokenizer)}")
+        print(f"🔍 [Thread {threading.current_thread().name}] Is tokenizer callable: {callable(self.model.tokenizer)}")
+        
+        # Check tokenizer attributes
+        if hasattr(self.model.tokenizer, '__dict__'):
+            print(f"🔍 [Thread {threading.current_thread().name}] Tokenizer attributes: {list(self.model.tokenizer.__dict__.keys())[:10]}")
+        
+        # Check if tokenizer has the methods we need
+        print(f"🔍 [Thread {threading.current_thread().name}] Has __call__: {hasattr(self.model.tokenizer, '__call__')}")
+        print(f"🔍 [Thread {threading.current_thread().name}] Has pad_token_id: {hasattr(self.model.tokenizer, 'pad_token_id')}")
+        print(f"🔍 [Thread {threading.current_thread().name}] Has eos_token_id: {hasattr(self.model.tokenizer, 'eos_token_id')}")
+        
+        try:
+            # Try to get pad_token_id directly
+            if hasattr(self.model.tokenizer, 'pad_token_id'):
+                print(f"🔍 [Thread {threading.current_thread().name}] pad_token_id value: {self.model.tokenizer.pad_token_id}")
+        except Exception as e:
+            print(f"🔍 [Thread {threading.current_thread().name}] Error getting pad_token_id: {e}")
+        
+        # Tokenize
+        print(f"🔍 [Thread {threading.current_thread().name}] Calling tokenizer now...")
+        try:
+            inputs = self.model.tokenizer(formatted_prompt, return_tensors="pt", padding=True, truncation=True)
+            print(f"🔍 [Thread {threading.current_thread().name}] Tokenization successful")
+        except Exception as e:
+            print(f"🔍 [Thread {threading.current_thread().name}] Tokenization failed with error: {e}")
+            print(f"🔍 [Thread {threading.current_thread().name}] Error type: {type(e)}")
+            import traceback
+            print(f"🔍 [Thread {threading.current_thread().name}] Full traceback:\n{traceback.format_exc()}")
+            raise
+        
+        inputs = {k: v.to(self.model.device) for k, v in inputs.items()}
+        
+        # Generate
+        with torch.no_grad():
+            # Get pad_token_id and eos_token_id safely
+            if hasattr(self.model.tokenizer, 'pad_token_id'):
+                pad_token_id = self.model.tokenizer.pad_token_id
+            elif isinstance(self.model.tokenizer, dict):
+                pad_token_id = self.model.tokenizer.get('pad_token_id', 0)
             else:
-                merged_config['suppress_tokens'] = think_tokens
+                pad_token_id = 0
+                
+            if hasattr(self.model.tokenizer, 'eos_token_id'):
+                eos_token_id = self.model.tokenizer.eos_token_id
+            elif isinstance(self.model.tokenizer, dict):
+                eos_token_id = self.model.tokenizer.get('eos_token_id', 0)
+            else:
+                eos_token_id = 0
+            
+            outputs = self.model.hf_model.generate(
+                **inputs,
+                max_new_tokens=merged_config.get('max_new_tokens', 150),
+                temperature=merged_config.get('temperature', 0.7),
+                top_p=merged_config.get('top_p', 0.9),
+                do_sample=True,
+                pad_token_id=pad_token_id,
+                eos_token_id=eos_token_id,
+            )
         
-        # Call model.generate with proper parameters
-        response, _ = self.model.generate(prompt, layer_index, **merged_config)
+        # Decode
+        response = self.model.tokenizer.decode(outputs[0][inputs['input_ids'].shape[1]:], skip_special_tokens=True)
         response = response.strip()
         
-        # Post-process to remove any thinking tags that might have slipped through
+        # Basic post-processing - only remove explicit think tags if they somehow appear
         import re
-        if self.suppress_thinking_tokens and '<think>' in response:
-            # Remove everything between <think> and </think> tags
+        if '<think>' in response:
+            # This shouldn't happen with enable_thinking=False, but just in case
             response = re.sub(r'<think>.*?</think>', '', response, flags=re.DOTALL)
             response = response.strip()
         
-        # Clean up any artifacts from suppressed tokens
-        if self.suppress_thinking_tokens:
-            # Remove strange characters that might appear at the start
-            response = re.sub(r'^[\.\$\s]+', '', response)
-            # Fix numbering that might be broken (e.g., ".$. 2." -> "1.")
-            lines = response.split('\n')
-            cleaned_lines = []
-            question_num = 1
-            for line in lines:
-                line = line.strip()
-                if line and re.match(r'^[\.\$\s]*\d+\.', line):
-                    # Replace the malformed number with correct numbering
-                    line = re.sub(r'^[\.\$\s]*\d+\.', f'{question_num}.', line)
-                    question_num += 1
-                cleaned_lines.append(line)
-            response = '\n'.join(cleaned_lines)
+        # No aggressive cleaning needed anymore since we're properly using enable_thinking=False
         
         # Update cache
         with self._cache_lock:
@@ -773,6 +817,7 @@ class SyntheticContrastivePairGenerator:
         pair_overgeneration_factor: float = 1.5,
         force_regenerate: bool = False,
         verbose_timing: bool = False,
+        name: Optional[str] = None,  # Added to fix bug where multiple callers pass this
     ) -> ContrastivePairSet:
         """
         Generate a complete contrastive pair set from a trait description.
@@ -784,6 +829,7 @@ class SyntheticContrastivePairGenerator:
             pair_overgeneration_factor: Factor to overgenerate pairs for diversity selection
             force_regenerate: If True, bypasses the cache and generates a new set.
             verbose_timing: If True, shows detailed timing for each step
+            name: Optional name for the pair set (used by various callers)
 
         Returns:
             ContrastivePairSet with generated pairs
@@ -955,8 +1001,10 @@ class SyntheticContrastivePairGenerator:
         )
         print(f"✅ Selected {len(diverse_pairs)} diverse pairs")
 
+        # Generate name from trait description
+        pair_set_name = f"synthetic_{trait_description[:30]}"
         pair_set: ContrastivePairSet = ContrastivePairSet(
-            name=f"synthetic_{trait_description[:30]}",
+            name=pair_set_name,
             task_type="synthetic",
             pairs=diverse_pairs,
         )
