@@ -305,16 +305,36 @@ class OptimizationPipeline:
         self, samples: list[dict], layer_id: int, dataset_name: str
     ) -> tuple[np.ndarray, np.ndarray]:
         """Create contrastive probe training data for a specific layer."""
+        self.logger.info(f"Creating probe data from {len(samples)} samples for {dataset_name} on layer {layer_id}")
+
         # Get task for the specified dataset
         task = get_task(dataset_name)
         extractor = task.get_extractor()
+        self.logger.debug(f"Using task: {task.__class__.__name__}, extractor: {extractor.__class__.__name__}")
 
         texts = []
         labels = []
+        success_count = 0
+        fail_count = 0
 
-        for sample in samples:
-            # Extract QA pair
-            contrastive_pair = extractor.extract_contrastive_pair(sample, task)
+        for i, sample in enumerate(samples):
+            try:
+                # Extract QA pair
+                contrastive_pair = extractor.extract_contrastive_pair(sample, task)
+
+                # Skip samples where contrastive pair extraction failed
+                if not contrastive_pair:
+                    self.logger.debug(f"Sample {i + 1}: No contrastive pair extracted from keys: {list(sample.keys())}")
+                    fail_count += 1
+                    continue
+
+                success_count += 1
+                self.logger.debug(f"Sample {i + 1}: Successfully extracted contrastive pair")
+
+            except Exception as e:
+                self.logger.error(f"Sample {i + 1}: Exception during contrastive pair extraction: {e}")
+                fail_count += 1
+                continue
 
             question = contrastive_pair["question"]
             correct_answer = contrastive_pair["correct_answer"]
@@ -332,6 +352,14 @@ class OptimizationPipeline:
             texts.append(incorrect_text)
             labels.append(0)
 
+        self.logger.info(
+            f"Probe data creation: {success_count} successful, {fail_count} failed. Generated {len(texts)} texts."
+        )
+
+        if len(texts) == 0:
+            self.logger.error("No texts generated for activation extraction! All contrastive pair extractions failed.")
+            return np.array([]), np.array([])
+
         activations = data_utils.extract_activations_with_hook(
             self.model, self.tokenizer, texts, layer_id, self.config.batch_size, self.config.max_length, self.device
         )
@@ -647,7 +675,7 @@ class OptimizationPipeline:
         # Save detailed validation results to JSON
         self._save_detailed_validation_results(questions, ground_truths, predictions, trial_number)
 
-        # Evaluate benchmark performance
+        # Evaluate benchmark performance using standard evaluation
         benchmark_metrics = metrics.evaluate_benchmark_performance(predictions, ground_truths, self.config.val_dataset)
 
         return benchmark_metrics.get("accuracy", 0.0)
@@ -888,11 +916,10 @@ class OptimizationPipeline:
                 test_questions, test_ground_truths, baseline_predictions, steered_predictions
             )
 
-        # Calculate benchmark metrics
+        # Calculate benchmark metrics using standard evaluation
         baseline_benchmark_metrics = metrics.evaluate_benchmark_performance(
             baseline_predictions, test_ground_truths, self.config.test_dataset
         )
-
         steered_benchmark_metrics = metrics.evaluate_benchmark_performance(
             steered_predictions, test_ground_truths, self.config.test_dataset
         )
@@ -996,18 +1023,40 @@ class OptimizationPipeline:
         detailed_results = []
 
         for i, (question, correct_answer, model_answer) in enumerate(zip(questions, ground_truths, predictions)):
-            # Evaluate if the answer is correct using metrics module
+            # Use standard evaluation via metrics module
             is_correct = metrics.evaluate_response_correctness(model_answer, correct_answer, self.config.val_dataset)
 
-            detailed_results.append(
-                {
-                    "row": i,
-                    "question": question,
-                    "correct_answer": correct_answer,
-                    "model_answer": model_answer,
-                    "is_correct": is_correct,
-                }
-            )
+            result_entry = {
+                "row": i,
+                "question": question,
+                "correct_answer": correct_answer,
+                "model_answer": model_answer,
+                "is_correct": is_correct,
+            }
+
+            # Add MC-specific fields if this is a multiple choice task
+            if self._should_use_multiple_choice_evaluation():
+                # Extract MC diagnostics directly without custom evaluation
+                import re
+
+                # Extract available answers from question (A. choice, B. choice, etc.)
+                available_answers = []
+                choice_pattern = r"([A-E])\.\s+(.+?)(?=\n[A-E]\.|$)"
+                matches = re.findall(choice_pattern, question, re.MULTILINE | re.DOTALL)
+                for letter, choice_text in matches:
+                    available_answers.append(f"{letter}. {choice_text.strip()}")
+
+                # Extract model's selected letter from model answer
+                model_selected_letter = "?"
+                model_letter_match = re.search(r"\b([A-E])\b", model_answer.upper())
+                if model_letter_match:
+                    model_selected_letter = model_letter_match.group(1)
+
+                result_entry["available_answers"] = available_answers
+                result_entry["correct_choice_letter"] = correct_answer
+                result_entry["model_selected_letter"] = model_selected_letter
+
+            detailed_results.append(result_entry)
 
         # Save to JSON file
         timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
@@ -1035,23 +1084,52 @@ class OptimizationPipeline:
         for i, (question, correct_answer, baseline_answer, steered_answer) in enumerate(
             zip(questions, ground_truths, baseline_predictions, steered_predictions)
         ):
-            # Evaluate correctness for both baseline and steered answers
+            # Use standard evaluation for both baseline and steered answers
             is_baseline_correct = metrics.evaluate_response_correctness(
                 baseline_answer, correct_answer, self.config.test_dataset
             )
             is_correct = metrics.evaluate_response_correctness(steered_answer, correct_answer, self.config.test_dataset)
 
-            detailed_results.append(
-                {
-                    "row": i,
-                    "question": question,
-                    "correct_answer": correct_answer,
-                    "baseline_model_answer": baseline_answer,
-                    "model_answer": steered_answer,
-                    "is_baseline_correct": is_baseline_correct,
-                    "is_correct": is_correct,
-                }
-            )
+            result_entry = {
+                "row": i,
+                "question": question,
+                "correct_answer": correct_answer,
+                "baseline_model_answer": baseline_answer,
+                "model_answer": steered_answer,
+                "is_baseline_correct": is_baseline_correct,
+                "is_correct": is_correct,
+            }
+
+            # Add MC-specific fields if this is a multiple choice task
+            if self._should_use_multiple_choice_evaluation():
+                # Extract MC diagnostics directly without custom evaluation
+                import re
+
+                # Extract available answers from question (A. choice, B. choice, etc.)
+                available_answers = []
+                choice_pattern = r"([A-E])\.\s+(.+?)(?=\n[A-E]\.|$)"
+                matches = re.findall(choice_pattern, question, re.MULTILINE | re.DOTALL)
+                for letter, choice_text in matches:
+                    available_answers.append(f"{letter}. {choice_text.strip()}")
+
+                # Extract steered model's selected letter
+                steered_selected_letter = "?"
+                steered_letter_match = re.search(r"\b([A-E])\b", steered_answer.upper())
+                if steered_letter_match:
+                    steered_selected_letter = steered_letter_match.group(1)
+
+                # Extract baseline model's selected letter
+                baseline_selected_letter = "?"
+                baseline_letter_match = re.search(r"\b([A-E])\b", baseline_answer.upper())
+                if baseline_letter_match:
+                    baseline_selected_letter = baseline_letter_match.group(1)
+
+                result_entry["available_answers"] = available_answers
+                result_entry["correct_choice_letter"] = correct_answer
+                result_entry["model_selected_letter"] = steered_selected_letter
+                result_entry["baseline_model_selected_letter"] = baseline_selected_letter
+
+            detailed_results.append(result_entry)
 
         # Save to JSON file
         timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
@@ -1134,6 +1212,13 @@ class OptimizationPipeline:
         if self.model is None:
             self._setup_experiment()
 
+        # Create timestamped run directory for evaluation-only mode
+        if not hasattr(self, "run_dir"):
+            self.run_timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
+            self.run_dir = self.output_dir / f"evaluate_only_{self.run_timestamp}"
+            self.run_dir.mkdir(parents=True, exist_ok=True)
+            self.logger.info(f"📁 Evaluation directory: {self.run_dir}")
+
         # Create a complete mock trial with all expected parameters
         from optuna.trial import FixedTrial
 
@@ -1332,6 +1417,11 @@ class OptimizationPipeline:
         except Exception as e:
             self.logger.warning(f"Failed to log final results to WandB: {e}")
 
+    def _should_use_multiple_choice_evaluation(self) -> bool:
+        """Determine if we should use multiple choice evaluation for this dataset."""
+        # Use multiple choice evaluation for TruthfulQA and other MC tasks
+        return self.config.test_dataset.lower() in ["truthfulqa_mc1", "truthfulqa", "mmlu"]
+
 
 def main():
     """Main entry point for optimization pipeline."""
